using UnityEngine;
using System;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;
using System.IO;

// IslDoorDutchEditor example
//
// Base editor for a CUA (Custom Unity Asset)
namespace IslDoorDutchEditorPlugin
{
    public class IslDoorDutchEditor : MVRScript
    {
        // My JSONStorable for the editor
        public JSONStorableFloat DoorTopSpring;
        public JSONStorableFloat DoorTopDamper;
		public JSONStorableFloat DoorBotSpring;
        public JSONStorableFloat DoorBotDamper;
		
    
        // My misc variables to find and edit the prefab made in Unity
        private string objectNameSearchString = "Isl_Door_Dutch";
        public Transform ObjectRoot;
        public Transform DoorTopRoot;
        public Transform DoorBotRoot;

        public HingeJoint DoorHingeTopJoint;
		public HingeJoint DoorHingeBotJoint;

        // Security and initialization
        private bool editorInitialized = false;
        private int initLoops = 0;
        private int maxInitLoops = 3600;

        // The usual init function for VAM
        public override void Init()
        {         
            try
            {
                // We only authorize the script to be used by a CUA
                if (containingAtom.type != "CustomUnityAsset")
                {
                    logError("IslDoorDutchEditor - Please add this script on a Custom Unity Asset with isloverlookextras.assetbundle loaded and Isl_Door_Dutch selected. ( script disabled ).");
                    return;
                }
            
                // Default color for both shapes
                HSVColor hsvc = HSVColorPicker.RGBToHSV(1f, 1f, 1f);
            
                // SpringTop
                DoorTopSpring = new JSONStorableFloat("Door Top Spring", 1f, DoorTopSpringCallback, 0f, 100f);
                CreateSlider(DoorTopSpring, false);
            
                // DamperTop
                DoorTopDamper = new JSONStorableFloat("Door Top Damper", 1f, DoorTopDamperCallback, 0f, 100f);
                CreateSlider(DoorTopDamper, false);
				
                // SpringBot
                DoorBotSpring = new JSONStorableFloat("Door Bot Spring", 1f, DoorBotSpringCallback, 0f, 100f);
                CreateSlider(DoorBotSpring, false);
            
                // DamperBot
                DoorBotDamper = new JSONStorableFloat("Door Bot Damper", 1f, DoorBotDamperCallback, 0f, 100f);
                CreateSlider(DoorBotDamper, false);
                
                // A tiny bit of help to give some info to your fellow creator and/or player
           JSONStorableString helpText = new JSONStorableString("Help",
                    "<color=#000><size=35><b>About</b></size></color>\n\n" +
                    "<color=#333>" +
                    "<b>IslDutchDoorEditor</b> This plugin allows you to adjust the Spring and Damper settings for the Top and Bottom Doors of the IslOverlook Extras Dutch Door Prop.\n\n" +
                    "Adjusting <b>Spring</b> will change the amount of  force the doors asserts to remain closed.\n\n" +
					"Adjusting <b>Damper</b> will change how quickly the doors close.\n\n" +
					"<b>Special Thanks</b> to Hazmhox for support in making this plugin possible.\n\n" +
                    "</color>"
                );
                UIDynamic helpTextfield = CreateTextField(helpText, true);
                helpTextfield.height = 1100.0f;
                
                RegisterFloat(DoorTopSpring);
                RegisterFloat(DoorTopDamper);
                RegisterFloat(DoorBotSpring);
                RegisterFloat(DoorBotDamper);				
				
            }
            catch(Exception e)
            {
                SuperController.LogError("IslDoorDutchEditor - Exception caught: " + e);
            }
        }
    
        // In this case we don't need that, but maybe you do ?
        void Start()
        {

        }
    
        // Used for the initialization of the editor.
        // When it is initialized, there's nothing done every frame
        void Update()
        {
            // If the editor is not initialized, we're gonna try to do it
            // I'm putting 3600 tries ( at 60fps it is 1min )
            if( initLoops < maxInitLoops && editorInitialized == false ) {
                initLoops++;
                InitEditor();
            }
        
            // Triggering an error if we have reached our limit of tries
            if( initLoops == maxInitLoops ) {
                logError("IslDoorDutchEditor : This CUA could not be initialized properly after 3600 tries. Please select a compatible object and reload the script.");
                initLoops++; // To avoid spamming the log with the error
            }
        }
        
        private bool InitEditor() 
		{
            // When the editor is initialized, we don't need to do that anymore
            if( editorInitialized == true ) return true;
        
            ObjectRoot = getObjectRoot( containingAtom.reParentObject );
            if( ObjectRoot == null) {
                return false;
            }
        
            // Finding my door
            DoorTopRoot = getChildNamed(ObjectRoot, "Door_Top");
			DoorBotRoot = getChildNamed(ObjectRoot, "Door_Bot");

            if( DoorTopRoot != null && DoorBotRoot != null) {
                DoorHingeTopJoint = DoorTopRoot.GetComponent<HingeJoint>();
                DoorHingeBotJoint = DoorBotRoot.GetComponent<HingeJoint>();
            
                if( DoorHingeTopJoint != null && DoorHingeBotJoint != null )
				{
                    // Everything is good, I can consider my editor initialized
                    editorInitialized = true;

                    InitDoorTopSettings();
                    InitDoorBotSettings();
                    return true;
                } else {
                    logError("IslDoorEditor : your CUA is not a door.");
                    initLoops = maxInitLoops; // We don't have what we need, we consider the initialization failed
                    return false;
                }
            } else {
                logError("IslDoorEditor : your CUA is not a door.");
                initLoops = maxInitLoops; // We don't have what we need, we consider the initialization failed
                return false;
			}
        }
    
        // Recursive function searching for a transform containing the string set in objectNameSearchString
        // It is used the first time for the initialization of the script
        private Transform getObjectRoot( Transform parent )
		{
            foreach( Transform child in parent )
            {
                if( child.name.Contains(objectNameSearchString) ) 
				{
                    return child;
                } else {
                    Transform childSearch = getObjectRoot(child);
                    if (childSearch != null) return childSearch;
                }
            }
        
            return null;
        }
        
        private Transform getChildNamed( Transform parent, string searchName ) {
            foreach( Transform child in parent )
            {
                if( child.name == searchName ) {
                    return child;
                } else {
                    Transform childSearch = getChildNamed(child,searchName);
                    if (childSearch != null) return childSearch;
                }
            }
        
            return null;
        }

        private void InitDoorTopSettings()
        {
            DoorTopSpringCallback(DoorTopSpring.val);
        }

        protected void DoorTopSpringCallback(float springTopVal) 
		{
            if( DoorHingeTopJoint )
            {
                JointSpring hingeSpring = DoorHingeTopJoint.spring;
                hingeSpring.spring = springTopVal;
                DoorHingeTopJoint.spring = hingeSpring;
            }
        }
        
        protected void DoorTopDamperCallback(float damperTopVal)
        {
            
            if( DoorHingeTopJoint )
            {
                JointSpring hingeSpring = DoorHingeTopJoint.spring;
                hingeSpring.damper = damperTopVal;
                DoorHingeTopJoint.spring = hingeSpring;
            }
        }
        
        private void InitDoorBotSettings()		
		
        {
           DoorBotSpringCallback(DoorBotSpring.val);
        }

        protected void DoorBotSpringCallback(float springBotVal)
		{
            if( DoorHingeBotJoint )
            {
                JointSpring hingeSpring = DoorHingeBotJoint.spring;
                hingeSpring.spring = springBotVal;
                DoorHingeBotJoint.spring = hingeSpring;
            }
        }
        
        protected void DoorBotDamperCallback(float damperBotVal)
        {
            
            if( DoorHingeBotJoint )
            {
                JointSpring hingeSpring = DoorHingeBotJoint.spring;
                hingeSpring.damper = damperBotVal;
                DoorHingeBotJoint.spring = hingeSpring;
            }
        }
		
        // **************************
        // Local Tools... just because I'm lazy
        // **************************
        private void logDebug( string debugText ) {
            SuperController.LogMessage( debugText );
        }
    
        private void logError( string debugText ) {
            SuperController.LogError( debugText );
        }
    }
}