using UnityEngine;
using System;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;
using System.IO;

//
//
// Base editor for a CUA (Custom Unity Asset)
namespace IslDoubleWindowEditorPlugin
{
    public class IslDoubleWindowEditor : MVRScript
    {
        // My JSONStorable for the editor
        public JSONStorableFloat WindowLeftSpring;
        public JSONStorableFloat WindowLeftDamper;
		public JSONStorableFloat WindowRightSpring;
        public JSONStorableFloat WindowRightDamper;
		
    
        // My misc variables to find and edit the prefab made in Unity
        private string objectNameSearchString = "Isl_Window_Double";
        public Transform ObjectRoot;
        public Transform WindowLeftRoot;
        public Transform WindowRightRoot;

        public HingeJoint WindowLeftJoint;
		public HingeJoint WindowRightJoint;

        // Security and initialization
        private bool editorInitialized = false;
        private int initLoops = 0;
        private int maxInitLoops = 3600;

        // The usual init function for VAM
        public override void Init()
        {         
            try
            {
                // We only authorize the script to be used by a CUA
                if (containingAtom.type != "CustomUnityAsset")
                {
                    logError("IslDoubleWindowEditor - Please add this script on a Custom Unity Asset with isloverlookextras.assetbundle loaded and Isl_Window_Double selected. ( script disabled ).");
                    return;
                }
            
                // Default color for both shapes
                HSVColor hsvc = HSVColorPicker.RGBToHSV(1f, 1f, 1f);
            
                // SpringTop
                WindowLeftSpring = new JSONStorableFloat("Window Left Spring", 1f, WindowLeftSpringCallback, 0f, 100f);
                CreateSlider(WindowLeftSpring, false);
            
                // DamperTop
                WindowLeftDamper = new JSONStorableFloat("Window Left Damper", 1f, WindowLeftDamperCallback, 0f, 100f);
                CreateSlider(WindowLeftDamper, false);
				
                // SpringBot
                WindowRightSpring = new JSONStorableFloat("Window Right Spring", 1f, WindowRightSpringCallback, 0f, 100f);
                CreateSlider(WindowRightSpring, false);
            
                // DamperBot
                WindowRightDamper = new JSONStorableFloat("Window Right Damper", 1f, WindowRightDamperCallback, 0f, 100f);
                CreateSlider(WindowRightDamper, false);
                
                // A tiny bit of help to give some info to your fellow creator and/or player
                JSONStorableString helpText = new JSONStorableString("Help",
                    "<color=#000><size=35><b>About</b></size></color>\n\n" +
                    "<color=#333>" +
                    "<b>IslDoubleWindowEditor</b> This plugin allows you to adjust the Spring and Damper settings for the Left and Right Windows of the IslOverlook Extras Double Window Prop.\n\n" +
                    "Adjusting <b>Spring</b> will change the amount of  force the window asserts to remain closed.\n\n" +
					"Adjusting <b>Damper</b> will change how quickly the windows close.\n\n" +
					"<b>Special Thanks</b> to Hazmhox for support in making this plugin possible.\n\n" +
                    "</color>"
                );
                UIDynamic helpTextfield = CreateTextField(helpText, true);
                helpTextfield.height = 1100.0f;
                
                RegisterFloat(WindowLeftSpring);
                RegisterFloat(WindowLeftDamper);
                RegisterFloat(WindowRightSpring);
                RegisterFloat(WindowRightDamper);				
				
            }
            catch(Exception e)
            {
                SuperController.LogError("IslDoubleWindowEditor - Exception caught: " + e);
            }
        }
    
        // In this case we don't need that, but maybe you do ?
        void Start()
        {

        }
    
        // Used for the initialization of the editor.
        // When it is initialized, there's nothing done every frame
        void Update()
        {
            // If the editor is not initialized, we're gonna try to do it
            // I'm putting 3600 tries ( at 60fps it is 1min )
            if( initLoops < maxInitLoops && editorInitialized == false ) {
                initLoops++;
                InitEditor();
            }
        
            // Triggering an error if we have reached our limit of tries
            if( initLoops == maxInitLoops ) {
                logError("IslDoubleWindowEditor : This CUA could not be initialized properly after 3600 tries. Please select a compatible object and reload the script.");
                initLoops++; // To avoid spamming the log with the error
            }
        }
        
        private bool InitEditor() 
		{
            // When the editor is initialized, we don't need to do that anymore
            if( editorInitialized == true ) return true;
        
            ObjectRoot = getObjectRoot( containingAtom.reParentObject );
            if( ObjectRoot == null) {
                return false;
            }
        
            // Finding my door
            WindowLeftRoot = getChildNamed(ObjectRoot, "Hinge_Right");
			WindowRightRoot = getChildNamed(ObjectRoot, "Hinge_left");

            if( WindowLeftRoot != null && WindowRightRoot != null) {
                WindowLeftJoint = WindowLeftRoot.GetComponent<HingeJoint>();
                WindowRightJoint = WindowRightRoot.GetComponent<HingeJoint>();
            
                if( WindowLeftJoint != null && WindowRightJoint != null )
				{
                    // Everything is good, I can consider my editor initialized
                    editorInitialized = true;

                    InitWindowLeftSettings();
                    InitWindowRightSettings();
                    return true;
                } else {
                    logError("IslDoubleWindowEditor : your CUA is not a Double Window.");
                    initLoops = maxInitLoops; // We don't have what we need, we consider the initialization failed
                    return false;
                }
            } else {
                logError("IslDoubleWindowEditor : your CUA is not a Double Window.");
                initLoops = maxInitLoops; // We don't have what we need, we consider the initialization failed
                return false;
			}
        }
    
        // Recursive function searching for a transform containing the string set in objectNameSearchString
        // It is used the first time for the initialization of the script
        private Transform getObjectRoot( Transform parent )
		{
            foreach( Transform child in parent )
            {
                if( child.name.Contains(objectNameSearchString) ) 
				{
                    return child;
                } else {
                    Transform childSearch = getObjectRoot(child);
                    if (childSearch != null) return childSearch;
                }
            }
        
            return null;
        }
        
        private Transform getChildNamed( Transform parent, string searchName ) {
            foreach( Transform child in parent )
            {
                if( child.name == searchName ) {
                    return child;
                } else {
                    Transform childSearch = getChildNamed(child,searchName);
                    if (childSearch != null) return childSearch;
                }
            }
        
            return null;
        }

        private void InitWindowLeftSettings()
        {
            WindowLeftSpringCallback(WindowLeftSpring.val);
        }

        protected void WindowLeftSpringCallback(float springTopVal) 
		{
            if( WindowLeftJoint )
            {
                JointSpring hingeSpring = WindowLeftJoint.spring;
                hingeSpring.spring = springTopVal;
                WindowLeftJoint.spring = hingeSpring;
            }
        }
        
        protected void WindowLeftDamperCallback(float damperTopVal)
        {
            
            if( WindowLeftJoint )
            {
                JointSpring hingeSpring = WindowLeftJoint.spring;
                hingeSpring.damper = damperTopVal;
                WindowLeftJoint.spring = hingeSpring;
            }
        }
        
        private void InitWindowRightSettings()		
		
        {
           WindowRightSpringCallback(WindowRightSpring.val);
        }

        protected void WindowRightSpringCallback(float springBotVal)
		{
            if( WindowRightJoint )
            {
                JointSpring hingeSpring = WindowRightJoint.spring;
                hingeSpring.spring = springBotVal;
                WindowRightJoint.spring = hingeSpring;
            }
        }
        
        protected void WindowRightDamperCallback(float damperBotVal)
        {
            
            if( WindowRightJoint )
            {
                JointSpring hingeSpring = WindowRightJoint.spring;
                hingeSpring.damper = damperBotVal;
                WindowRightJoint.spring = hingeSpring;
            }
        }
		
        // **************************
        // Local Tools... just because I'm lazy
        // **************************
        private void logDebug( string debugText ) {
            SuperController.LogMessage( debugText );
        }
    
        private void logError( string debugText ) {
            SuperController.LogError( debugText );
        }
    }
}